<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class FuncionariosHierarqFactory {
    
    use TQuery;

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    // Busca toda a ramificação de usuários da hierarquias
    public function getHierarchyDetails($nrhierarquiafunc) {
        $sql = "SELECT A.NRFUNCIONARIOSHIERARQ, A.NRFUNCIONARIO, A.NRFUNCIONARIOPAI, B.NMFUNCIONARIO, H.NMHIERARQUIAFUNC
                    FROM FCM_FUNCIONARIOSHIERARQ A
                    JOIN FCM_FUNCIONARIO B ON B.NRFUNCIONARIO = A.NRFUNCIONARIO
                    JOIN FCM_HIERARQUIAFUNC H ON H.NRHIERARQUIAFUNC = A.NRHIERARQUIAFUNC
                    WHERE A.NRHIERARQUIAFUNC = {$nrhierarquiafunc}";

        $std = $this->connection->prepare($sql);

        $std->execute();
        return $std->fetchAll();
    }

    public function getHierarquiaFacilities($parametros) {

        $sql = "SELECT A.NRFUNCIONARIOSHIERARQ, A.NRFUNCIONARIO, A.NRFUNCIONARIOPAI, B.NMFUNCIONARIO, H.NMHIERARQUIAFUNC
                    FROM FCM_FUNCIONARIOSHIERARQ A
                        JOIN FCM_FUNCIONARIO B ON B.NRFUNCIONARIO = A.NRFUNCIONARIO
                        JOIN FCM_HIERARQUIAFUNC H ON H.NRHIERARQUIAFUNC = A.NRHIERARQUIAFUNC
                    WHERE A.NRHIERARQUIAFUNC = (SELECT NRHIERARQUIAFUNC
                                                FROM FCM_HIERARQUIAFUNC
                                                    WHERE IDATIVO = 'S')
                    AND A.NRFUNCIONARIO IN (SELECT NRFUNCIONARIO
                                                        FROM FCM_FUNCIONARIOSHIERARQ
                                                    START WITH NRFUNCIONARIO     = :NRFUNCIONARIO
                                                    CONNECT BY PRIOR NRFUNCIONARIO = NRFUNCIONARIOPAI)";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
        return $std->fetchAll();
    }
    
    // Busca uma lista personalizada de usuários que podem ser adicionados na hierarquia
    public function getHierarchyUsers($nrhierarquiafunc) {
        $sql = " SELECT NRFUNCIONARIO, NMFUNCIONARIO
                    FROM FCM_FUNCIONARIO
                  WHERE NRFUNCIONARIO NOT IN( SELECT A.NRFUNCIONARIO
                                                            FROM FCM_FUNCIONARIO A
                                                                JOIN FCM_FUNCIONARIOSHIERARQ B ON  B.NRFUNCIONARIO = A.NRFUNCIONARIO
                                                WHERE B.NRHIERARQUIAFUNC = {$nrhierarquiafunc})";

        $std = $this->connection->prepare($sql);

        $std->execute();
        return $std->fetchAll();
    }
   
   // Busca uma lista personalizada de usuários que podem ser Pais na hierarquia 
    public function getHierarchyUsersParents($nrhierarquiafunc) {
        $sql = "SELECT NRFUNCIONARIO, NMFUNCIONARIO
                    FROM FCM_FUNCIONARIO
                  WHERE NRFUNCIONARIO IN( SELECT A.NRFUNCIONARIO
                                                            FROM FCM_FUNCIONARIO A
                                                                JOIN FCM_FUNCIONARIOSHIERARQ B ON  B.NRFUNCIONARIO = A.NRFUNCIONARIO
                                              WHERE B.NRHIERARQUIAFUNC = {$nrhierarquiafunc})";

        $std = $this->connection->prepare($sql);

        $std->execute();
        return $std->fetchAll();
    }
    
    // Insere um novo usuário à hierarquia
    public function insertHierarchyUser($params) {
        $sql = "INSERT INTO FCM_FUNCIONARIOSHIERARQ (NRFUNCIONARIOSHIERARQ, NRHIERARQUIAFUNC, NRFUNCIONARIO,
                                                    NRFUNCIONARIOPAI, DTINCLUSAO)
                    VALUES (:NRFUNCIONARIOSHIERARQ, :NRHIERARQUIAFUNC, :NRFUNCIONARIO,
                            :NRFUNCIONARIOPAI, SYSDATE)";

        $std = $this->connection->prepare($sql);

        $std->execute($params);
    }
    
    // Deleta um usuário e sua ramificação abaixo
    public function deleteHierarchyUser($nrfuncionario) {
        $sql = "DELETE FROM FCM_FUNCIONARIOSHIERARQ
                    WHERE NRFUNCIONARIO = {$nrfuncionario}";

        $std = $this->connection->prepare($sql);

        $std->execute();
    }
}
?>