/* eslint-disable */
import {
 Http, IDictionary, Metadata, Loader,
} from '@zeedhi/core';
import {
  AlertService, DialogService, Form, LoadingService, Modal, ModalService, Select, Text,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
export class TabBankDetailsController {
  private addBankDetails!: Modal;

  private editionBankDetails!: Modal;

  private NRCONTCORRPARC: any;

  public BPController = Loader.getInstance('BusinessPartnerController');

  public setFilterGrid() {
    const NRPARCNEGOCIO = window.location.search.split('?')[1];
    const grid = Metadata.getInstance<TekGrid>('tekgrid-tabAddBankCommunication');
    grid.datasource.setFilter({
      NRPARCNEGOCIO: NRPARCNEGOCIO,
    });
    grid.datasource.get();
  }

  public alertMessages(message: string, type: string) {
    AlertService.show({
      name: 'basic-alert',
      text: message,
      top: true,
      left: false,
      timeout: 3000,
      color: type,
    });
  }

  public dialogMessage(message: string) {
    DialogService.show({
      name: 'warning-dialog',
      type: 'warning',
      title: 'Erro!',
      text: message,
    });
  }

  public showDeleteDialog({ event } : any) {
    event.preventDefault();

    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'Excluir Endereço',
      text: 'Todas as informações serão perdidas se concluir está ação. Deseja continuar?',
      buttons: [
        {
          name: 'button-no',
          component: 'ZdButton',
          label: 'Não',
          events: {
            click: () => DialogService.hide(),
          },
        },
        {
          name: 'button-yes',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: '{{TabBankDetailsController.deleteBankDetails}}'
          },
          outline: true,
        },
      ],
    });
  }

  public async deleteBankDetails() {
    const grid = Metadata.getInstance<TekGrid>('tekgrid-tabAddBankCommunication');
    const NRS = (grid.selectedRows.map((element: any) => element.NRCONTCORRPARC)).toString();

    await Http.post('/deleteBankDetails', {
      NRCONTCORRPARC: NRS,
      NRPARCNEGOCIO: window.location.search.split('?')[1],
    }).then((response: IDictionary) => {
      if (response.data.dataset.bank[0] === 'OK') {
        this.alertMessages('Dados Bancários excluídos com sucesso', 'success');
        grid.datasource.get();
        grid.selectedRows = [];
        DialogService.hide();
      } else {
        this.dialogMessage('Operação bloqueada! Este Dado Bancário não pôde ser adicionado.<br><br>Se o problema persistir entre em contato com o suporte.')
      }
    });
  }

  public async openModalBankDetails() {
    this.addBankDetails = await ModalService.createFromJson('BusinessPartner/Modal/modal-add-bankDetails', true);
    this.addBankDetails.show();
  }

  public closeModalBankDetails() {
    this.addBankDetails.hide();
    this.addBankDetails?.destroy();
  }

  public onMountNameModal() {
    Metadata.getInstance<Text>('nameOfCorporate-addBankDetails').text = this.BPController.NAMEOFCORPORATE;
  }

  public async changeAgencyToEnable({ component }: any) {
    const selectAgency = Metadata.getInstance<Select>('formSelectAgency');

    if (!component.value) {
      selectAgency.disabled = true;
      selectAgency.setValue(null);
    } else {
      selectAgency.disabled = false;
      selectAgency.datasource.setFilter({
        CDBANCO: component.value,
      });
    }
    await this.changeCheckingAccountToEnable();
  }

  public insertNewBankDetails() {
    const formBank = Metadata.getInstance<Form>('form-addBankDetails');
    const grid = Metadata.getInstance<TekGrid>('tekgrid-tabAddBankCommunication');

    if (formBank.validate()) {
      LoadingService.show('Salvando informações...');
      Http.post('/insertNewBankDetails', {
        NRPARCNEGOCIO: window.location.search.split('?')[1],
        CDBANCO: Metadata.getInstance<Select>('formSelectBank').value,
        CDAGENCIA: Metadata.getInstance<Select>('formSelectAgency').value,
        CDCONTCORR: Metadata.getInstance<Select>('formSelectCheckingAccount').value,
        NRTPCONTCORR: Metadata.getInstance<Select>('formSelectTypeOfCheckingAccount').value,
        CDOPERINCLUSAO: localStorage.getItem('CDOPERADOR'),
      }).then((response: IDictionary) => {
        const returnMessage = response.data.dataset.bank[0];
        if (returnMessage === 'OK') {
          this.alertMessages('Dados Bancários adicionados com sucesso!', 'success');
          this.closeModalBankDetails();
          grid.datasource.get();
        } else {
          this.closeModalBankDetails();
          this.dialogMessage('Operação bloqueada! Estes Dados Bancários não puderam ser inseridos.<br><br>Se o problema persistir entre em contato com o suporte.');
        }
      });
      LoadingService.hide();
    } else {
      this.alertMessages('Por favor, preencha todos os campos obrigatórios(*)!', 'error');
    }
  }

  public changeCheckingAccountToEnable() {
    const selectAgency = Metadata.getInstance<Select>('formSelectAgency');
    const selectBank = Metadata.getInstance<Select>('formSelectBank');
    const selectCheckingAccount = Metadata.getInstance<Select>('formSelectCheckingAccount');

    if (!selectAgency.value) {
      selectCheckingAccount.disabled = true;
      selectCheckingAccount.setValue(null);
    } else {
      selectCheckingAccount.disabled = false;
      selectCheckingAccount.datasource.setFilter({
        CDBANCO: selectBank.value,
        CDAGENCIA: selectAgency.value,
      });
    }
  }

  // Funções do Modal de Edição
  public async openModalEdition({ row }: any) {
    this.editionBankDetails = await ModalService.createFromJson('BusinessPartner/Modal/modal-edition-bankDetails', true);
    this.NRCONTCORRPARC = row.NRCONTCORRPARC;
    this.editionBankDetails.show();
  }

  public closeModalEdition() {
    this.editionBankDetails.hide();
    this.editionBankDetails?.destroy();
  }

  public async onMountBankDetails() {
    const NRCONTCORRPARC = this.NRCONTCORRPARC;
    const NAMEOFCORPORATE = this.BPController.NAMEOFCORPORATE;
    console.log(NAMEOFCORPORATE);
    let response1: any;

    await Http.post('/getSpecificBankDetails', { NRCONTCORRPARC }).then((response: IDictionary) => {
      response1 = response.data.dataset.detailsFound;
    });
    response1.forEach(async (data: IDictionary) => {
      Metadata.getInstance<Text>('nameOfCorporate-editionBankDetails').text = `${data.NMBANCO} - ${data.NMAGENCIA} << ${NAMEOFCORPORATE}`
      Metadata.getInstance<Text>('NRCONTCORRPARC').text = NRCONTCORRPARC;
      await Metadata.getInstance<Select>('formEdition-bank').setValue(data.CDBANCO);
      await Metadata.getInstance<Select>('formEdition-agency').setValue(data.CDAGENCIA);
      await Metadata.getInstance<Select>('formEdition-checkingAccount').setValue(data.CDCONTCORR);
      Metadata.getInstance<Select>('formEdition-typeOfCheckingAccount').setValue(data.NRTPCONTCORR);
    });
    await this.changeEditionAgencyToEnable();
  }

  public async changeEditionAgencyToEnable() {
    const selectBank = Metadata.getInstance<Select>('formEdition-bank');
    const selectAgency = Metadata.getInstance<Select>('formEdition-agency');

    if (!selectBank.value) {
      selectAgency.disabled = true;
      selectAgency.setValue(null);
    } else {
      selectAgency.disabled = false;
      selectAgency.datasource.setFilter({
        CDBANCO: selectBank.value,
      });
    }
    await this.changeEditionCheckingAccountToEnable();
  }

  public changeEditionCheckingAccountToEnable() {
    const selectBank = Metadata.getInstance<Select>('formEdition-bank');
    const selectAgency = Metadata.getInstance<Select>('formEdition-agency');
    const selectCheckingAccount = Metadata.getInstance<Select>('formEdition-checkingAccount');

    if (!selectAgency.value) {
      selectCheckingAccount.disabled = true;
      selectCheckingAccount.setValue(null);
    } else {
      selectCheckingAccount.disabled = false;
      selectCheckingAccount.datasource.setFilter({
        CDBANCO: selectBank.value,
        CDAGENCIA: selectAgency.value,
      });
    }
  }

  public updateBankDetails() {
    const formBank = Metadata.getInstance<Form>('formEdition-bankDetails');
    const grid = Metadata.getInstance<TekGrid>('tekgrid-tabAddBankCommunication');

    if (formBank.validate()) {
      LoadingService.show('Salvando alterações...');
      Http.post('/updateBankDetails', {
        NRCONTCORRPARC: Metadata.getInstance<Text>('NRCONTCORRPARC').text,
        NRPARCNEGOCIO: window.location.search.split('?')[1],
        CDBANCO: Metadata.getInstance<Select>('formEdition-bank').value,
        CDAGENCIA: Metadata.getInstance<Select>('formEdition-agency').value,
        CDCONTCORR: Metadata.getInstance<Select>('formEdition-checkingAccount').value,
        NRTPCONTCORR: Metadata.getInstance<Select>('formEdition-typeOfCheckingAccount').value,
        CDOPERULTATU: localStorage.getItem('CDOPERADOR'),
      }).then((response: IDictionary) => {
        const returnMessage = response.data.dataset.bank[0];
        if (returnMessage === 'OK') {
          this.alertMessages('Dados Bancários alterados com sucesso!', 'success');
          this.closeModalEdition();
          grid.datasource.get();
        } else {
          this.closeModalEdition();
          this.dialogMessage('Operação bloqueada! Estes Dados Bancários não podem ser alterados.<br><br>Se o problema persistir entre em contato com o suporte.');
        }
      });
      LoadingService.hide();
    } else {
      this.alertMessages('Por favor, preencha todos os campos obrigatórios(*)!', 'error');
    }
  }

}
